from typing import Dict, Any, Sequence
import logging

from mcp import Tool
from mcp.types import TextContent

from .base import BaseHandler
from mysql_mcp_server_pro.exception.exceptions import SQLExecutionError
from ..utils.execute_sql_util import ExecuteSqlUtil


logger = logging.getLogger(__name__)

class ExecuteSQL(BaseHandler):
    """SQL 执行处理器"""
    
    name = "execute_sql"
    description = "在MySQL数据库上执行SQL (支持多条SQL语句，以分号分隔)"

    def get_tool_description(self) -> Tool:
        """获取工具描述"""
        return Tool(
            name=self.name,
            description=self.description,
            inputSchema={
                "type": "object",
                "properties": {
                    "query": {
                        "type": "string",
                        "description": "要执行的SQL语句"
                    }
                },
                "required": ["query"]
            }
        )

    async def run_tool(self, arguments: Dict[str, Any]) -> Sequence[TextContent]:
        """执行SQL工具
        
        Args:
            arguments: 包含SQL查询的参数字典
            
        Returns:
            执行结果文本序列
        """
        if "query" not in arguments:
            return [TextContent(type="text", text="错误: 缺少查询语句")]

        query = arguments["query"]
        
        try:
            exe = ExecuteSqlUtil()
            sql_results = exe.execute_multiple_statements(query)
            
            # 格式化结果
            results = []
            for result in sql_results:
                formatted_result = exe.format_result(result)
                results.append(formatted_result)
            
            return [TextContent(type="text", text="\n---\n".join(results))]
            
        except SQLExecutionError as e:
            return [TextContent(type="text", text=str(e))]


from typing import Dict, Any

from mcp import GetPromptResult
from mcp.types import Prompt, TextContent, PromptMessage

from mysql_mcp_server_pro.prompts.BasePrompt import BasePrompt


class AnalysisMySqlIssues(BasePrompt):
    name = "analyzing-mysql-prompt"
    description = (
        "这是分析mysql相关问题的提示词"
    )

    def get_prompt(self) -> Prompt:
        return Prompt(
            name= self.name,
            description= self.description,
            arguments=[
            ],
        )

    async def run_prompt(self, arguments: Dict[str, Any]) -> GetPromptResult:

        prompt = """- Role: 数据库运维专家和MySQL问题诊断工程师"
        - Background: 用户在使用MySQL数据库时遇到了问题，需要对问题进行深入分析，以便快速定位并解决，确保数据库的稳定运行和数据的完整性。"
        - Profile: 你是一位资深的数据库运维专家，对MySQL数据库的架构、性能优化、故障排查有着丰富的实战经验，熟悉Linux操作系统和SQL语言，能够运用专业的工具和方法进行问题诊断。
        - Skills: 你具备数据库性能监控、日志分析、SQL语句优化、备份与恢复等关键能力，能够熟练使用MySQL的命令行工具、配置文件以及第三方监控工具。
        - Goals: 快速定位MySQL数据库问题的根源，提供有效的解决方案，恢复数据库的正常运行，保障数据安全。
        - Constrains: 在解决问题的过程中，应遵循最小干预原则，避免对现有数据造成不必要的影响，确保解决方案的可操作性和安全性。
        - OutputFormat: 以问题分析报告的形式输出，包括问题描述、分析过程、解决方案和预防措施。
        - Workflow:
          1. 收集问题相关信息，包括错误日志、系统配置、SQL语句执行情况等。
          2. 分析问题产生的可能原因，从数据库性能、配置错误、SQL语句逻辑等方面进行排查。
          3. 提出针对性的解决方案，并进行测试验证，确保问题得到解决。
        - Examples:
          - 例子1：数据库连接失败
            问题描述：用户无法连接到MySQL数据库，提示“无法连接到服务器”。
            分析过程：检查MySQL服务是否正常运行，查看错误日志是否有相关记录，确认网络连接是否正常，检查数据库的用户权限和连接限制。
            解决方案：启动MySQL服务，修复网络问题，调整用户权限，增加连接限制。
            预防措施：定期检查MySQL服务状态，监控网络连接，合理配置用户权限。
          - 例子2：查询性能低下
            问题描述：执行SQL查询语句时响应时间过长。
            分析过程：查看执行计划，检查表的索引是否合理，分析SQL语句是否存在优化空间，检查数据库的缓存配置。
            解决方案：优化SQL语句，创建合适的索引，调整缓存参数。
            预防措施：定期分析和优化SQL语句，监控数据库性能，及时调整索引和缓存配置。
          - 例子3：数据丢失
            问题描述：部分数据丢失，无法恢复。
            分析过程：检查备份策略是否正常执行，查看二进制日志是否完整，分析是否有误操作导致数据丢失。
            解决方案：从备份中恢复数据，利用二进制日志进行数据恢复，分析误操作并采取补救措施。
            预防措施：制定完善的备份策略，定期检查备份文件的完整性和可用性，加强用户操作权限管理。
        - Initialization: 在第一次对话中，请直接输出以下：您好，作为数据库运维专家，我将协助您分析和解决MySQL数据库的问题。请详细描述您遇到的问题，包括错误信息、操作步骤等，以便我更好地进行分析。           
        """

        return GetPromptResult(
            description="mysql prompt",
            messages=[
                PromptMessage(
                    role="user",
                    content=TextContent(type="text", text=prompt),
                )
            ],
        )


from typing import Dict, Sequence, Any

from mcp import Tool
from mcp.types import TextContent

from .base import BaseHandler
from mysql_mcp_server_pro.config import get_db_config
from mysql_mcp_server_pro.handles import (
    ExecuteSQL
)


class GetTableDesc(BaseHandler):
    name = "get_table_desc"
    description = (
        "根据表名搜索数据库中对应的表字段,支持多表查询(Search for table structures in the database based on table names, supporting multi-table queries)"
    )

    def get_tool_description(self) -> Tool:
        return Tool(
            name=self.name,
            description=self.description,
            inputSchema={
                "type": "object",
                "properties": {
                    "text": {
                        "type": "string",
                        "description": "要搜索的表名"
                    }
                },
                "required": ["text"]
            }
        )

    async def run_tool(self, arguments: Dict[str, Any]) -> Sequence[TextContent]:
            """获取指定表的字段结构信息

            参数:
                text (str): 要查询的表名，多个表名以逗号分隔

            返回:
                list[TextContent]: 包含查询结果的TextContent列表
                - 返回表的字段名、字段注释等信息
                - 结果按表名和字段顺序排序
                - 结果以CSV格式返回，包含列名和数据
            """
            try:
                if "text" not in arguments:
                    raise ValueError("缺少查询语句")

                text = arguments["text"]

                config = get_db_config()
                execute_sql = ExecuteSQL()

                # 将输入的表名按逗号分割成列表
                table_names = [name.strip() for name in text.split(',')]
                # 构建IN条件
                table_condition = "','".join(table_names)
                sql = "SELECT TABLE_NAME, COLUMN_NAME, COLUMN_COMMENT "
                sql += f"FROM information_schema.COLUMNS WHERE TABLE_SCHEMA = '{config['database']}' "
                sql += f"AND TABLE_NAME IN ('{table_condition}') ORDER BY TABLE_NAME, ORDINAL_POSITION;"
                return await execute_sql.run_tool({"query":sql})

            except Exception as e:
                return [TextContent(type="text", text=f"执行查询时出错: {str(e)}")]


import os
from dotenv import load_dotenv

def get_db_config():
    """从环境变量获取数据库配置信息

    返回:
        dict: 包含数据库连接所需的配置信息
        - host: 数据库主机地址
        - port: 数据库端口
        - user: 数据库用户名
        - password: 数据库密码
        - database: 数据库名称
        - role: 数据库角色权限

    异常:
        ValueError: 当必需的配置信息缺失时抛出
    """
    # 加载.env文件
    ##load_dotenv()

    config = {
        "host": os.getenv("MYSQL_HOST", "localhost"),
        "port": int(os.getenv("MYSQL_PORT", "3306")),
        "user": os.getenv("MYSQL_USER"),
        "password": os.getenv("MYSQL_PASSWORD"),
        "database": os.getenv("MYSQL_DATABASE"),
        "role": os.getenv("MYSQL_ROLE", "readonly"),  # 默认为只读角色
        "pool_size": int(os.getenv("POOL_SIZE", 10)),
        "max_overflow": int(os.getenv("POOL_MAX_OVERFLOW", 20)),
        "pool_recycle": int(os.getenv("POOL_RECYCLE", 3600)),
        "pool_timeout": int(os.getenv("POOL_TIMEOUT", 30))
    }

    if not all([config["user"], config["password"], config["database"]]):
        raise ValueError("缺少必需的数据库配置")

    return config

# 定义角色权限
ROLE_PERMISSIONS = {
    "readonly": ["SELECT", "SHOW", "DESCRIBE", "EXPLAIN"],  # 只读权限
    "writer": ["SELECT", "SHOW", "DESCRIBE", "EXPLAIN", "INSERT", "UPDATE", "DELETE"],  # 读写权限
    "admin": ["SELECT", "SHOW", "DESCRIBE", "EXPLAIN", "INSERT", "UPDATE", "DELETE", 
             "CREATE", "ALTER", "DROP", "TRUNCATE"]  # 管理员权限
}

def get_role_permissions(role: str) -> list:
    """获取指定角色的权限列表
    
    参数:
        role (str): 角色名称
        
    返回:
        list: 该角色允许执行的SQL操作列表
    """
    return ROLE_PERMISSIONS.get(role, ROLE_PERMISSIONS["readonly"])  # 默认返回只读权限


from typing import Dict, Any, Sequence

from mcp import Tool
from mcp.types import TextContent

from .base import BaseHandler

from mysql_mcp_server_pro.handles import (
    ExecuteSQL
)

execute_sql = ExecuteSQL()

class GetDBHealthRunning(BaseHandler):
    name = "get_db_health_running"
    description = (
        "获取当前mysql的健康状态(Analyze MySQL health status )"
    )

    def get_tool_description(self) -> Tool:
        return Tool(
            name=self.name,
            description=self.description,
            inputSchema={
                "type": "object",
                "properties": {

                }
            }
        )

    async def run_tool(self, arguments: Dict[str, Any]) -> Sequence[TextContent]:
        lock_result = await self.get_lock(arguments)
        processlist_result = await self.get_processlist(arguments)
        status_result = await self.get_status(arguments)
        trx_result = await self.get_trx(arguments)


        # 合并结果
        combined_result = []
        combined_result.extend(processlist_result)
        combined_result.extend(lock_result)
        combined_result.extend(trx_result)
        combined_result.extend(status_result)

        return combined_result

    """
        获取连接情况
    """
    async def get_processlist(self, arguments: Dict[str, Any]) -> Sequence[TextContent]:
        try:
            sql = "SHOW FULL PROCESSLIST;SHOW VARIABLES LIKE 'max_connections';"

            return await execute_sql.run_tool({"query": sql})
        except Exception as e:
            return [TextContent(type="text", text=f"执行查询时出错: {str(e)}")]

    """
        获取运行情况
    """
    async def get_status(self, arguments: Dict[str, Any]) -> Sequence[TextContent]:
        try:
            sql = "SHOW ENGINE INNODB STATUS;"

            return await execute_sql.run_tool({"query": sql})
        except Exception as e:
            return [TextContent(type="text", text=f"执行查询时出错: {str(e)}")]

    """
        获取事务情况
    """
    async def get_trx(self, arguments: Dict[str, Any]) -> Sequence[TextContent]:
        try:
            sql = "SELECT * FROM INFORMATION_SCHEMA.INNODB_TRX;"
            return await execute_sql.run_tool({"query": sql})
        except Exception as e:
            return [TextContent(type="text", text=f"执行查询时出错: {str(e)}")]


    """
        获取锁情况
    """
    async def get_lock(self, arguments: Dict[str, Any]) -> Sequence[TextContent]:
        try:
            sql = "SHOW OPEN TABLES WHERE In_use > 0;select * from information_schema.innodb_locks;select * from information_schema.innodb_lock_waits;"
            sql += "select * from performance_schema.data_lock_waits;"
            sql += "select * from performance_schema.data_locks;"


            return await execute_sql.run_tool({"query": sql})
        except Exception as e:
            return [TextContent(type="text", text=f"执行查询时出错: {str(e)}")]


"""
SQLAlchemy数据库连接池工具类
支持多种数据库类型，提供统一的连接池管理接口
"""

import logging
from contextlib import contextmanager
from typing import Dict, Any, Optional
from sqlalchemy import create_engine, text
from sqlalchemy.engine import Engine
from sqlalchemy.pool import QueuePool, SingletonThreadPool, NullPool
from sqlalchemy.exc import SQLAlchemyError

# 配置日志
logger = logging.getLogger(__name__)


class SQLAlchemyConnectionPool:
    """
    基于SQLAlchemy的数据库连接池实现
    """

    def __init__(self, database_url: str, 
                 pool_type: str = "queue",
                 pool_size: int = 10,
                 max_overflow: int = 20,
                 pool_recycle: int = 3600,
                 pool_pre_ping: bool = True,
                 pool_timeout: int = 30,
                 **kwargs):
        """
        初始化连接池
        
        Args:
            database_url: 数据库连接URL
            pool_type: 连接池类型 ('queue', 'singleton', 'null')
            pool_size: 连接池大小
            max_overflow: 超出pool_size后最多可创建的连接数
            pool_recycle: 连接回收时间(秒)，-1表示不回收
            pool_pre_ping: 是否在使用前ping数据库以检查连接有效性
            pool_timeout: 获取连接的超时时间(秒)
            **kwargs: 其他传递给create_engine的参数
        """
        self.database_url = database_url
        self.pool_type = pool_type
        self.pool_size = pool_size
        self.max_overflow = max_overflow
        self.pool_recycle = pool_recycle
        self.pool_pre_ping = pool_pre_ping
        self.pool_timeout = pool_timeout
        
        # 创建引擎
        self.engine = self._create_engine(**kwargs)
        
        logger.info(f"SQLAlchemy connection pool initialized for {database_url}")
        logger.info(f"Pool type: {pool_type}, Pool size: {pool_size}, Max overflow: {max_overflow}")

    def _create_engine(self, **kwargs) -> Engine:
        """
        创建SQLAlchemy引擎
        
        Returns:
            Engine: SQLAlchemy引擎实例
        """
        # 根据类型选择连接池
        pool_class_map = {
            'queue': QueuePool,
            'singleton': SingletonThreadPool,
            'null': NullPool
        }
        
        pool_class = pool_class_map.get(self.pool_type, QueuePool)
        
        # 创建引擎
        engine = create_engine(
            self.database_url,
            poolclass=pool_class,
            pool_size=self.pool_size,
            max_overflow=self.max_overflow,
            pool_recycle=self.pool_recycle,
            pool_pre_ping=self.pool_pre_ping,
            pool_timeout=self.pool_timeout,
            **kwargs
        )
        
        return engine

    def get_connection(self):
        """
        获取数据库连接
        
        Returns:
            数据库连接对象
        """
        try:
            conn = self.engine.connect()
            logger.debug("Database connection acquired from pool")
            return conn
        except SQLAlchemyError as e:
            logger.error(f"Failed to acquire database connection: {e}")
            raise

    def return_connection(self, conn):
        """
        归还数据库连接到连接池
        
        Args:
            conn: 数据库连接对象
        """
        try:
            conn.close()
            logger.debug("Database connection returned to pool")
        except SQLAlchemyError as e:
            logger.warning(f"Error returning connection to pool: {e}")

    @contextmanager
    def connection(self):
        """
        上下文管理器方式使用连接
        
        Usage:
            with pool.connection() as conn:
                # 使用连接执行数据库操作
                result = conn.execute(text("SELECT 1"))
        """
        conn = self.get_connection()
        try:
            yield conn
        finally:
            self.return_connection(conn)

    def execute_query(self, query: str, params: Optional[Dict] = None):
        """
        执行查询语句
        
        Args:
            query: SQL查询语句
            params: 查询参数
            
        Returns:
            查询结果
        """
        with self.connection() as conn:
            try:
                result = conn.execute(text(query), params or {})
                return result.fetchall()
            except SQLAlchemyError as e:
                logger.error(f"Query execution failed: {e}")
                raise

    def execute_non_query(self, query: str, params: Optional[Dict] = None):
        """
        执行非查询语句（INSERT, UPDATE, DELETE等）
        
        Args:
            query: SQL语句
            params: 查询参数
            
        Returns:
            影响的行数
        """
        with self.connection() as conn:
            try:
                result = conn.execute(text(query), params or {})
                conn.commit()
                return result.rowcount
            except SQLAlchemyError as e:
                conn.rollback()
                logger.error(f"Non-query execution failed: {e}")
                raise

    def get_stats(self) -> Dict[str, Any]:
        """
        获取连接池统计信息
        
        Returns:
            包含连接池统计信息的字典
        """
        pool = self.engine.pool
        return {
            "pool_type": self.pool_type,
            "pool_size": self.pool_size,
            "max_overflow": self.max_overflow,
            "checked_out_connections": pool.checkedout(),
            "available_connections": pool.checkedin(),
            "overflow_connections": getattr(pool, 'overflow', 0),
            "recycle_time": self.pool_recycle
        }

    def close_all_connections(self):
        """
        关闭所有连接
        """
        try:
            self.engine.dispose()
            logger.info("All database connections closed")
        except Exception as e:
            logger.error(f"Error closing all connections: {e}")


def create_mysql_pool(host: str, port: int = 3306, user: str = "root", 
                      password: str = "", database: str = "",
                      pool_type: str = "queue",
                      pool_size: int = 10,
                      max_overflow: int = 20,
                      pool_recycle: int = 3600,
                      **kwargs) -> SQLAlchemyConnectionPool:
    """
    创建MySQL连接池
    
    Args:
        host: 数据库主机
        port: 数据库端口
        user: 用户名
        password: 密码
        database: 数据库名
        pool_type: 连接池类型
        pool_size: 连接池大小
        max_overflow: 最大溢出连接数
        pool_recycle: 连接回收时间
        **kwargs: 其他参数
        
    Returns:
        SQLAlchemyConnectionPool: MySQL连接池实例
    """
    # 构建MySQL连接URL
    database_url = f"mysql+pymysql://{user}:{password}@{host}:{port}/{database}"
    
    return SQLAlchemyConnectionPool(
        database_url=database_url,
        pool_type=pool_type,
        pool_size=pool_size,
        max_overflow=max_overflow,
        pool_recycle=pool_recycle,
        **kwargs
    )



from datetime import datetime, timedelta
import jwt
from typing import Dict, Optional, Tuple

from ..oauth import oauth_config


class TokenHandler:
    @staticmethod
    def create_tokens(user_id: str, username: str) -> Tuple[str, str, datetime, datetime]:
        """
        创建访问令牌和刷新令牌
        
        Args:
            user_id: 用户ID
            username: 用户名
            
        Returns:
            Tuple[str, str, datetime, datetime]: (访问令牌, 刷新令牌, 访问令牌过期时间, 刷新令牌过期时间)
        """
        # 计算过期时间
        access_token_expires = datetime.utcnow() + timedelta(minutes=oauth_config.ACCESS_TOKEN_EXPIRE_MINUTES)
        refresh_token_expires = datetime.utcnow() + timedelta(days=oauth_config.REFRESH_TOKEN_EXPIRE_DAYS)
        
        # 访问令牌数据
        access_token_data = {
            "sub": str(user_id),  # subject (用户ID)
            "username": username,
            "type": "access_token",
            "exp": access_token_expires,
            "iat": datetime.utcnow()
        }
        
        # 刷新令牌数据
        refresh_token_data = {
            "sub": str(user_id),
            "username": username,
            "type": "refresh_token",
            "exp": refresh_token_expires,
            "iat": datetime.utcnow()
        }
        
        # 生成令牌
        access_token = jwt.encode(
            access_token_data,
            oauth_config.TOKEN_SECRET_KEY,
            algorithm=oauth_config.TOKEN_ALGORITHM
        )
        
        refresh_token = jwt.encode(
            refresh_token_data,
            oauth_config.TOKEN_SECRET_KEY,
            algorithm=oauth_config.TOKEN_ALGORITHM
        )
        
        return access_token, refresh_token, access_token_expires, refresh_token_expires
    
    @staticmethod
    def verify_token(token: str) -> Optional[Dict]:
        """
        验证令牌
        
        Args:
            token: JWT令牌
            
        Returns:
            Optional[Dict]: 令牌payload，无效则返回None
        """
        try:
            payload = jwt.decode(
                token,
                oauth_config.TOKEN_SECRET_KEY,
                algorithms=[oauth_config.TOKEN_ALGORITHM]
            )
            return payload
        except jwt.InvalidTokenError:
            return None
    
    @staticmethod
    def create_token_response(
        access_token: str,
        refresh_token: str,
        access_token_expires: datetime,
        refresh_token_expires: datetime
    ) -> Dict:
        """
        创建标准的OAuth2.0令牌响应
        
        Args:
            access_token: 访问令牌
            refresh_token: 刷新令牌
            access_token_expires: 访问令牌过期时间
            refresh_token_expires: 刷新令牌过期时间
            
        Returns:
            Dict: OAuth2.0标准令牌响应
        """
        # 转换为北京时间 (UTC+8)
        access_token_expires_beijing = access_token_expires + timedelta(hours=8)
        refresh_token_expires_beijing = refresh_token_expires + timedelta(hours=8)
        
        return {
            "access_token": access_token,
            "token_type": "bearer",
            "expires_in": oauth_config.ACCESS_TOKEN_EXPIRE_MINUTES * 60,  # 秒
            "refresh_token": refresh_token,
            "refresh_token_expires_in": oauth_config.REFRESH_TOKEN_EXPIRE_DAYS * 24 * 60 * 60,  # 秒
            "expire_time": access_token_expires_beijing.strftime("%Y-%m-%d %H:%M:%S (北京时间)"),
            "refresh_token_expire_time": refresh_token_expires_beijing.strftime("%Y-%m-%d %H:%M:%S (北京时间)")
        } 


import asyncio
import contextlib
import os

from collections.abc import AsyncIterator
from starlette.responses import Response, HTMLResponse

import click
import uvicorn

from typing import Sequence, Dict, Any
from mcp.server.sse import SseServerTransport

from mcp.server.lowlevel import Server
from mcp.server.streamable_http_manager import StreamableHTTPSessionManager
from mcp.types import Tool, TextContent, Prompt, GetPromptResult

from starlette.applications import Starlette
from starlette.routing import Route, Mount
from starlette.types import Scope, Receive, Send
from starlette.middleware import Middleware

from .config import get_db_config
from .utils.execute_sql_util import ExecuteSqlUtil
from .config.event_store import InMemoryEventStore
from .handles.base import ToolRegistry
from .prompts.BasePrompt import PromptRegistry
from .oauth import OAuthMiddleware, login, login_page





# 初始化服务器
app = Server("operateMysql")

@app.list_prompts()
async def handle_list_prompts() -> list[Prompt]:
    """获取所有可用的提示模板列表
    
    返回:
        list[Prompt]: 返回系统中注册的所有提示模板列表
    """
    return PromptRegistry.get_all__prompts()


@app.get_prompt()
async def handle_get_prompt(name: str, arguments: Dict[str, Any] | None) -> GetPromptResult:
    """获取并执行指定的提示模板
    
    参数:
        name (str): 提示模板的名称
        arguments (dict[str, str] | None): 提示模板所需的参数字典，可以为空
        
    返回:
        GetPromptResult: 提示模板执行的结果
        
    说明:
        1. 根据提供的模板名称从注册表中获取对应的提示模板
        2. 使用提供的参数执行该模板
        3. 返回执行结果
    """
    prompt = PromptRegistry.get_prompt(name)
    return await prompt.run_prompt(arguments)


@app.list_tools()
async def list_tools() -> list[Tool]:
    """
        列出所有可用的MySQL操作工具
    """
    return ToolRegistry.get_all_tools()

@app.call_tool()
async def call_tool(name: str, arguments: Dict[str, Any]) -> Sequence[TextContent]:
    """调用指定的工具执行操作
    
    Args:
        name (str): 工具名称
        arguments (dict): 工具参数

    Returns:
        Sequence[TextContent]: 工具执行结果

    Raises:
        ValueError: 当指定了未知的工具名称时抛出异常
    """
    tool = ToolRegistry.get_tool(name)

    return await tool.run_tool(arguments)


async def run_stdio():
    """运行标准输入输出模式的服务器
    
    使用标准输入输出流(stdio)运行服务器，主要用于命令行交互模式
    
    Raises:
        Exception: 当服务器运行出错时抛出异常
    """
    from mcp.server.stdio import stdio_server

    async with stdio_server() as (read_stream, write_stream):
        try:
            await app.run(
                read_stream,
                write_stream,
                app.create_initialization_options()
            )
        except Exception as e:
            print(f"服务器错误: {str(e)}")
            raise

def run_sse():
    """运行SSE(Server-Sent Events)模式的服务器
    
    启动一个支持SSE的Web服务器，允许客户端通过HTTP长连接接收服务器推送的消息
    服务器默认监听0.0.0.0:9000
    """
    sse = SseServerTransport("/messages/")

    async def handle_sse(request):
        """处理SSE连接请求
        
        Args:
            request: HTTP请求对象
        """
        async with sse.connect_sse(
                request.scope, request.receive, request._send
        ) as streams:
            await app.run(streams[0], streams[1], app.create_initialization_options())
        return Response(status_code=204)  

    starlette_app = Starlette(
        debug=True,
        routes=[
            Route("/sse", endpoint=handle_sse),
            Mount("/messages/", app=sse.handle_post_message)
        ],
    )
    uvicorn.run(starlette_app, host="0.0.0.0", port=9000)

def run_streamable_http(json_response: bool, oauth: bool):
    event_store = InMemoryEventStore()

    session_manager = StreamableHTTPSessionManager(
        app=app,
        event_store=event_store,
        json_response=json_response,
    )

    async def handle_streamable_http(
            scope: Scope, receive: Receive, send: Send
    ) -> None:
        if scope["type"] == "lifespan":
            while True:
                message = await receive()
                if message["type"] == "lifespan.startup":
                    await send({"type": "lifespan.startup.complete"})
                elif message["type"] == "lifespan.shutdown":
                    await send({"type": "lifespan.shutdown.complete"})
                    return
        else:
            await session_manager.handle_request(scope, receive, send)

    @contextlib.asynccontextmanager
    async def lifespan(app: Starlette) -> AsyncIterator[None]:
        async with session_manager.run():
            yield

    routes = []

    middleware = []

    if oauth:
        middleware.append(
            Middleware(OAuthMiddleware, exclude_paths=["/login", "/mcp/auth/login"])
        )
        routes.append(Route("/login", endpoint=login_page, methods=["GET"]))
        routes.append(Route("/mcp/auth/login", endpoint=login, methods=["POST"]))

    routes.append(Mount("/mcp", app=handle_streamable_http))

    # 创建应用实例
    starlette_app = Starlette(
        debug=True,
        routes=routes,
        middleware=middleware,
        lifespan=lifespan
    )

    config = uvicorn.Config(
        app=starlette_app,
        host="0.0.0.0",
        port=3000,
        lifespan="on"
    )

    server = uvicorn.Server(config)
    server.run()

@click.command()
@click.option("--envfile", default=None, help="env file path")
@click.option("--mode", default="streamable_http", help="mode type")
@click.option("--oauth", default=False, help="open oauth")
def main(mode, envfile, oauth):
    """
    主入口函数，用于命令行启动
    支持三种模式：
    1. SSE 模式：mysql-mcp-server
    2. stdio 模式：mysql-mcp-server --stdio
    3. streamable http 模式（默认）
    
    Args:
        mode (str): 运行模式，可选值为 "sse" 或 "stdio"
    """
    from dotenv import load_dotenv

    # 优先加载指定的env文件
    if envfile:
        load_dotenv(envfile)
    else:
        # 获取当前文件（server.py）所在目录的绝对路径
        BASE_DIR = os.path.dirname(os.path.abspath(__file__))
        # 拼接出 config/.env 的绝对路径
        env_path = os.path.join(BASE_DIR, "config", ".env")
        load_dotenv(env_path)

    ExecuteSqlUtil.create_mysql_pool(db_config=get_db_config())

    #from .config.dbconfig import get_db_config
    #config = get_db_config()
    #print("============",config)

    # 使用传入的默认模式
    if mode == "stdio":
        asyncio.run(run_stdio())
    elif mode == "sse":
        run_sse()
    else:
        run_streamable_http(False,oauth)

if __name__ == "__main__":
    main()



from typing import Dict, Sequence, Any

from mcp import Tool
from mcp.types import TextContent

from .base import BaseHandler

"""

#背景
    - 该工具仅做研究探索使用
    - 主要探索mcp是否能够形成一条链式，根据内置定义的prompt自动调用所需的所有工具，使得模型更精准的调用。

#效果
    - 模型确实能够形成链式进行调用
    - 具有局限性，即只能针对某种情况进行定义，固定的要求它调用指定的工具
    
#使用方式
    - 该工具不作为常用的固定工具，如同样想要尝试链式的，可以将该工具继承BaseHandler类，
    即将下面的class UsePromptQueryTableData() 改为 class UsePromptQueryTableData(BaseHandler) 
    
# Background
    - This tool is for research and exploration purposes only.
    - The main goal is to explore whether MCP can form a chain, automatically invoking all required tools based on built-in prompt definitions, 
    enabling the model to call tools more accurately.

# Effect
    - The model can indeed perform chained invocations.
    - There are limitations: it can only be defined for certain scenarios, requiring fixed rules to call specified tools.

# Usage
    - This tool is not intended as a regular fixed tool. If you also want to experiment with chaining, you can inherit this tool from the BaseHandler class,
    i.e., change the following class UsePromptQueryTableData() to class UsePromptQueryTableData(BaseHandler)
"""

class UsePromptQueryTableData():
    name = "use_prompt_queryTableData"
    description = (
        "查询表中的数据信息的提示词，根据需求在调用对应工具返回结果，（Retrieve data records from the database table.）"
    )

    def get_tool_description(self) -> Tool:
        return Tool(
            name=self.name,
            description=self.description,
            inputSchema={
                "type": "object",
                "properties": {

                }
            }
        )

    async def run_tool(self, arguments: Dict[str, Any]) -> Sequence[TextContent]:
        prompt = f"""
                - Workflow:
                  1. 解析用户输入的自然语言指令，提取关键信息，如表描述和查询条件。
                  2. 判断是否跨库查询、是否明确指定了目标表名（例如是中文的描述、英文的描述，偏向语义化的描述则判断为未明确表名）
                  3. 未明确指定目标表名则调用“get_table_name”工具，获取对应的表名。
                  4. 调用“get_table_desc”工具，获取表的结构信息。
                  5. 根据表结构信息和用户输入的查询条件，生成SQL查询语句并调用“execute_sql”工具，返回查询结果。
                - Examples:
                  - 例子1：用户输入“查询用户表张三的数据”
                    解析结果：表描述为“用户表”，查询条件为“张三”。
                    判断结果：1.没有出现跨库的情况 2.未明确指定表名，当前为表的描述，需调用工具获取表名
                    调用工具“get_table_name”：根据“用户表”描述获取表名，假设返回表名为“user_table”。
                    调用工具“get_table_desc”：根据“user_table”获取表结构，假设表结构包含字段“id”、“name”、“age”。
                    生成SQL查询语句：`SELECT * FROM user_table WHERE name = '张三';`
                    调用工具“execute_sql”：根据生成的SQL,获取结果。
                    查询结果：返回张三的相关数据。
                - task: 
                  - 调用工具“get_table_name”，
                  - 调用工具“get_table_desc”，
                  - 调用工具“execute_sql”
                  - 以markdown格式返回执行结果
                """

        return [TextContent(type="text", text=prompt)]


from .execute_sql import ExecuteSQL
from .get_chinese_initials import GetChineseInitials
from .get_table_desc import GetTableDesc
from .get_table_index import GetTableIndex
from .get_table_lock import GetTableLock
from .get_table_name import GetTableName
from .get_db_health_running import GetDBHealthRunning
from .get_db_health_index_usage import GetDBHealthIndexUsage
from .use_prompt_queryTableData import UsePromptQueryTableData
from .optimize_sql import OptimizeSql

__all__ = [
    "ExecuteSQL",
    "GetChineseInitials",
    "GetTableDesc",
    "GetTableIndex",
    "GetTableLock",
    "GetTableName",
    "GetDBHealthRunning",
    "GetDBHealthIndexUsage",
    "UsePromptQueryTableData",
    "OptimizeSql"
]


from typing import Dict, Any, Sequence

from mcp import Tool
from mcp.types import TextContent

from .base import BaseHandler
from mysql_mcp_server_pro.config import get_db_config
from mysql_mcp_server_pro.handles import (
    ExecuteSQL
)

class GetTableIndex(BaseHandler):
    name = "get_table_index"
    description = (
        "根据表名搜索数据库中对应的表索引,支持多表查询(Search for table indexes in the database based on table names, supporting multi-table queries)"
    )

    def get_tool_description(self) -> Tool:
        return Tool(
            name=self.name,
            description=self.description,
            inputSchema={
                "type": "object",
                "properties": {
                    "text": {
                        "type": "string",
                        "description": "要搜索的表名"
                    }
                },
                "required": ["text"]
            }
        )

    async def run_tool(self, arguments: Dict[str, Any]) -> Sequence[TextContent]:
        """获取指定表的索引信息

        参数:
            text (str): 要查询的表名，多个表名以逗号分隔

        返回:
            list[TextContent]: 包含查询结果的TextContent列表
            - 返回表的索引名、索引字段、索引类型等信息
            - 结果按表名、索引名和索引顺序排序
            - 结果以CSV格式返回，包含列名和数据
        """
        try:
            if "text" not in arguments:
                raise ValueError("缺少查询语句")

            text = arguments["text"]

            config = get_db_config()
            execute_sql = ExecuteSQL()

            # 将输入的表名按逗号分割成列表
            table_names = [name.strip() for name in text.split(',')]
            # 构建IN条件
            table_condition = "','".join(table_names)

            sql = "SELECT TABLE_NAME, INDEX_NAME, COLUMN_NAME, SEQ_IN_INDEX, NON_UNIQUE, INDEX_TYPE "
            sql += f"FROM information_schema.STATISTICS WHERE TABLE_SCHEMA = '{config['database']}' "
            sql += f"AND TABLE_NAME IN ('{table_condition}') ORDER BY TABLE_NAME, INDEX_NAME, SEQ_IN_INDEX;"

            return await execute_sql.run_tool({"query": sql})

        except Exception as e:
            return [TextContent(type="text", text=f"执行查询时出错: {str(e)}")]


from typing import Dict, Any, Sequence, Type, ClassVar

from mcp.types import TextContent, Tool


class ToolRegistry:
    """工具注册表，用于管理所有工具实例"""
    _tools: ClassVar[Dict[str, 'BaseHandler']] = {}

    @classmethod
    def register(cls, tool_class: Type['BaseHandler']) -> Type['BaseHandler']:
        """注册工具类
        
        Args:
            tool_class: 要注册的工具类
            
        Returns:
            返回注册的工具类，方便作为装饰器使用
        """
        tool = tool_class()
        cls._tools[tool.name] = tool
        return tool_class

    @classmethod
    def get_tool(cls, name: str) -> 'BaseHandler':
        """获取工具实例
        
        Args:
            name: 工具名称
            
        Returns:
            工具实例
            
        Raises:
            ValueError: 当工具不存在时抛出
        """
        if name not in cls._tools:
            raise ValueError(f"未知的工具: {name}")
        return cls._tools[name]

    @classmethod
    def get_all_tools(cls) -> list[Tool]:
        """获取所有工具的描述
        
        Returns:
            所有工具的描述列表
        """
        return [tool.get_tool_description() for tool in cls._tools.values()]


class BaseHandler:
    """工具基类"""
    name: str = ""
    description: str = ""

    def __init_subclass__(cls, **kwargs):
        """子类初始化时自动注册到工具注册表"""
        super().__init_subclass__(**kwargs)
        if cls.name:  # 只注册有名称的工具
            ToolRegistry.register(cls)

    def get_tool_description(self) -> Tool:
        raise NotImplementedError

    async def run_tool(self, arguments: Dict[str, Any]) -> Sequence[TextContent]:
        raise NotImplementedError




from typing import Dict, Any, Sequence

from mcp import Tool
from mcp.types import TextContent

from .base import BaseHandler
from mysql_mcp_server_pro.config import get_db_config
from mysql_mcp_server_pro.handles import (
    ExecuteSQL
)


class GetTableName(BaseHandler):

    name = "get_table_name"
    description = (
        "根据表中文名或表描述搜索数据库中对应的表名(Search for table names in the database based on table comments and descriptions )"
    )

    def get_tool_description(self) -> Tool:
        return Tool(
            name=self.name,
            description=self.description,
            inputSchema={
                "type": "object",
                "properties": {
                    "text": {
                        "type": "string",
                        "description": "要搜索的表中文名、表描述，仅支持单个查询"
                    }
                },
                "required": ["text"]
            }
        )

    async def run_tool(self, arguments: Dict[str, Any]) -> Sequence[TextContent]:
            """根据表的注释搜索数据库中的表名

            参数:
                text (str): 要搜索的表中文注释关键词

            返回:
                list[TextContent]: 包含查询结果的TextContent列表
                - 返回匹配的表名、数据库名和表注释信息
                - 结果以CSV格式返回，包含列名和数据
            """
            try:
                if "text" not in arguments:
                    raise ValueError("缺少查询语句")

                text = arguments["text"]

                config = get_db_config()
                execute_sql = ExecuteSQL()

                sql = "SELECT TABLE_SCHEMA, TABLE_NAME, TABLE_COMMENT "
                sql += f"FROM information_schema.TABLES WHERE TABLE_SCHEMA = '{config['database']}' AND TABLE_COMMENT LIKE '%{text}%';"
                return await execute_sql.run_tool({"query":sql})

            except Exception as e:
                return [TextContent(type="text", text=f"执行查询时出错: {str(e)}")]


from typing import Optional
from pydantic_settings import BaseSettings
from datetime import timedelta

class OAuthConfig(BaseSettings):
    """OAuth配置类"""
    
    # 客户端配置
    CLIENT_ID: str = "mysql-mcp-client"
    CLIENT_SECRET: str = "mysql-mcp-secret"  # 使用一个固定的密钥
    
    # Token配置
    ACCESS_TOKEN_EXPIRE_MINUTES: int = 30
    REFRESH_TOKEN_EXPIRE_DAYS: int = 30
    
    # Token加密配置
    TOKEN_SECRET_KEY: str = "mysql-mcp-server-pro-secret-key"  # 使用一个固定的密钥
    TOKEN_ALGORITHM: str = "HS256"
    
    # 允许的授权类型
    GRANT_TYPES: list[str] = ["password", "refresh_token"]
    
    class Config:
        env_file = ".env"
        case_sensitive = True

# 创建配置实例
oauth_config = OAuthConfig() 


from typing import Dict, Any, Sequence

from mcp import Tool
from mcp.types import TextContent

from .base import BaseHandler
from mysql_mcp_server_pro.config import get_db_config
from mysql_mcp_server_pro.handles import (
    ExecuteSQL
)

execute_sql = ExecuteSQL()

class GetDBHealthIndexUsage(BaseHandler):
    name = "get_db_health_index_usage"
    description = (
        "获取当前连接的mysql库的索引使用情况,包含冗余索引情况、性能较差的索引情况、未使用索引且查询时间大于30秒top5情况"
        + "(Get the index usage of the currently connected mysql database, including redundant index situations, "
        +  "poorly performing index situations, and the top 5 unused index situations with query times greater than 30 seconds)"
    )

    def get_tool_description(self) -> Tool:
        return Tool(
            name=self.name,
            description=self.description,
            inputSchema={
                "type": "object",
                "properties": {

                }
            }
        )

    async def run_tool(self, arguments: Dict[str, Any]) -> Sequence[TextContent]:
        config = get_db_config()

        count_zero_result = await self.get_count_zero(arguments,config)
        max_time_result = await self.get_max_timer(arguments,config)
        not_used_index_result = await self.get_not_used_index(arguments,config)


        # 合并结果
        combined_result = []
        combined_result.extend(count_zero_result)
        combined_result.extend(max_time_result)
        combined_result.extend(not_used_index_result)


        return combined_result

    """
        获取冗余索引情况
    """
    async def get_count_zero(self, arguments: Dict[str, Any], config) -> Sequence[TextContent]:
        try:
            sql = "SELECT object_name,index_name,count_star from performance_schema.table_io_waits_summary_by_index_usage "
            sql += f"WHERE object_schema = '{config['database']}' and count_star = 0 AND sum_timer_wait = 0 ;"

            return await execute_sql.run_tool({"query": sql})
        except Exception as e:
            return [TextContent(type="text", text=f"执行查询时出错: {str(e)}")]


    """
        获取性能较差的索引情况
    """
    async def get_max_timer(self, arguments: Dict[str, Any], config) -> Sequence[TextContent]:
        try:
            sql = "SELECT object_schema,object_name,index_name,(max_timer_wait / 1000000000000) max_timer_wait "
            sql += f"FROM performance_schema.table_io_waits_summary_by_index_usage where object_schema = '{config['database']}' "
            sql += "and index_name is not null ORDER BY  max_timer_wait DESC;"

            return await execute_sql.run_tool({"query": sql})
        except Exception as e:
            return [TextContent(type="text", text=f"执行查询时出错: {str(e)}")]

    """
        获取未使用索引查询时间大于30秒的top5情况
    """
    async def get_not_used_index(self, arguments: Dict[str, Any], config) -> Sequence[TextContent]:
        try:
            sql = "SELECT object_schema,object_name, (max_timer_wait / 1000000000000) max_timer_wait "
            sql += f"FROM performance_schema.table_io_waits_summary_by_index_usage where object_schema = '{config['database']}' "
            sql += "and index_name IS null and max_timer_wait > 30000000000000 ORDER BY max_timer_wait DESC limit 5;"

            return await execute_sql.run_tool({"query": sql})
        except Exception as e:
            return [TextContent(type="text", text=f"执行查询时出错: {str(e)}")]


import os
import hashlib
import time

from starlette.responses import JSONResponse, HTMLResponse
from starlette.requests import Request
from pathlib import Path

from .token_handler import TokenHandler
from .config import oauth_config


async def login_page(request: Request) -> HTMLResponse:
    """返回登录页面"""
    try:
        templates_dir = Path(__file__).parent.parent / "templates"
        login_html = templates_dir / "login.html"
        
        with open(login_html, "r", encoding="utf-8") as f:
            content = f.read()
        
        return HTMLResponse(content)
    except Exception as e:
        return HTMLResponse(
            content=f"<h1>错误</h1><p>加载登录页面失败: {str(e)}</p>",
            status_code=500
        )

async def login(request: Request) -> JSONResponse:
    """
    OAuth 2.0密码模式端点
    
    请求格式：
    POST /mcp/auth/login
    Content-Type: application/json
    {
        "grant_type": "password",
        "username": "用户名",
        "password": "密码",
        "client_id": "客户端ID",
        "client_secret": "客户端密钥"
    }
    """
    # 检查 Accept 头部
    accept_header = request.headers.get("accept", "*/*")
    if accept_header != "*/*" and "application/json" not in accept_header:
        return JSONResponse(
            {"error": "not_acceptable", "error_description": "Client must accept application/json response"},
            status_code=406
        )

    try:
        data = await request.json()
        
        # 验证授权类型
        grant_type = data.get("grant_type")
        if grant_type not in oauth_config.GRANT_TYPES:
            return JSONResponse(
                {"error": "unsupported_grant_type"},
                status_code=400
            )
            
        # 验证客户端凭据
        client_id = data.get("client_id")
        client_secret = data.get("client_secret")
        
        if not client_id or not client_secret:
            return JSONResponse(
                {"error": "invalid_client"},
                status_code=401
            )
            
        if client_id != oauth_config.CLIENT_ID or client_secret != oauth_config.CLIENT_SECRET:
            return JSONResponse(
                {"error": "invalid_client"},
                status_code=401
            )
        
        if grant_type == "password":
            username = data.get("username")
            #password = data.get("password")

            encrypted_password = data.get("password")  # 从前端接收的加密密码

            if not username or not encrypted_password:
                return JSONResponse(
                    {"error": "invalid_request", "error_description": "Missing username or password"},
                    status_code=400
                )

            # 获取时间戳和盐值
            timestamp = request.headers.get("X-Timestamp")
            salt = request.headers.get("X-Salt")

            if not timestamp or not salt:
                return JSONResponse(
                    {"error": "invalid_request", "error_description": "Missing security parameters"},
                    status_code=400
                )

            # 验证时间戳是否在有效期内（5分钟）
            try:
                ts = int(timestamp) / 1000  # 转换为秒
                current_time = time.time()
                if current_time - ts > 300:  # 5分钟过期
                    return JSONResponse(
                        {"error": "invalid_request", "error_description": "Request expired"},
                        status_code=400
                    )
            except (ValueError, TypeError):
                return JSONResponse(
                    {"error": "invalid_request", "error_description": "Invalid timestamp"},
                    status_code=400
                )

            # 验证密码
            # 1. 使用与前端相同的加密方式验证
            # 第一次哈希：密码 + 盐
            first_hash = hashlib.sha256((os.getenv("OAUTH_USER_PASSWORD", "admin") + salt).encode()).hexdigest()
            # 第二次哈希：第一次哈希结果 + 时间戳
            expected_hash = hashlib.sha256((first_hash + timestamp).encode()).hexdigest()


            # 这里应该添加实际的用户验证逻辑
            if username == os.getenv("OAUTH_USER_NAME", "admin") and encrypted_password == expected_hash :
                # 生成令牌
                access_token, refresh_token, access_expires, refresh_expires = TokenHandler.create_tokens(
                    user_id="1",  # 这里应该是实际的用户ID
                    username=username
                )
                
                # 返回标准的OAuth2.0响应
                return JSONResponse(
                    TokenHandler.create_token_response(
                        access_token,
                        refresh_token,
                        access_expires,
                        refresh_expires
                    )
                )
            
            return JSONResponse(
                {"error": "invalid_grant"},
                status_code=401
            )
            
        elif grant_type == "refresh_token":
            refresh_token = data.get("refresh_token")
            if not refresh_token:
                return JSONResponse(
                    {"error": "invalid_request"},
                    status_code=400
                )
                
            # 验证刷新令牌
            payload = TokenHandler.verify_token(refresh_token)
            if not payload or payload.get("type") != "refresh_token":
                return JSONResponse(
                    {"error": "invalid_grant"},
                    status_code=401
                )
                
            # 生成新的令牌
            access_token, new_refresh_token, access_expires, refresh_expires = TokenHandler.create_tokens(
                user_id=payload["sub"],
                username=payload["username"]
            )
            
            return JSONResponse(
                TokenHandler.create_token_response(
                    access_token,
                    new_refresh_token,
                    access_expires,
                    refresh_expires
                )
            )
    
    except Exception as e:
        return JSONResponse(
            {"error": "server_error", "error_description": str(e)},
            status_code=500
        ) 


"""
Utils package for mysql_mcp_server_pro
"""

# 导出数据库连接池工具
from .database_pool import SQLAlchemyConnectionPool, create_mysql_pool

__all__ = [
    "SQLAlchemyConnectionPool",
    "create_mysql_pool"

]


from typing import ClassVar, Dict, Type, Any

from mcp import GetPromptResult
from mcp.types import Prompt

class PromptRegistry:
    """prompt注册表，用于管理所有prompt实例"""
    _prompts: ClassVar[Dict[str, 'BasePrompt']] = {}

    @classmethod
    def register(cls, prompt_class: Type['BasePrompt']) -> Type['BasePrompt']:
        """注册prompt类

        Args:
            prompt_class: 要注册的工具类

        Returns:
            返回注册的prompt类，方便作为装饰器使用
        """
        prompt = prompt_class()
        cls._prompts[prompt.name] = prompt
        return prompt_class

    @classmethod
    def get_prompt(cls, name: str) -> 'BasePrompt':
        """获取prompt实例

        Args:
            name: prompt名称

        Returns:
            prompt实例

        Raises:
            ValueError: 当prompt不存在时抛出
        """
        if name not in cls._prompts:
            raise ValueError(f"未知的prompt: {name}")
        return cls._prompts[name]

    @classmethod
    def get_all__prompts(cls) -> list[Prompt]:
        """获取所有prompt的描述

        Returns:
            所有prompt的描述列表
        """
        return [prompt.get_prompt() for prompt in cls._prompts.values()]


class BasePrompt:
    name:str = ""
    description:str = ""

    def __init_subclass__(cls, **kwargs):
        """子类初始化时自动注册到prompt注册表"""
        super().__init_subclass__(**kwargs)
        if cls.name:  # 只注册有名称的prompt
            PromptRegistry.register(cls)

    def get_prompt(self) -> Prompt:
        raise NotImplementedError()

    async def run_prompt(self, arguments: Dict[str, Any]) -> GetPromptResult:
        raise NotImplementedError()



from typing import Dict, Any, Sequence

from mcp import Tool
from mcp.types import TextContent
from pypinyin import pinyin, Style
from .base import BaseHandler


class GetChineseInitials(BaseHandler):
    name = "get_chinese_initials"
    description = (
        "创建表结构时，将中文字段名转换为拼音首字母字段"
    )

    def get_tool_description(self) -> Tool:
        return Tool(
            name=self.name,
            description=self.description,
            inputSchema={
                "type": "object",
                "properties": {
                    "text": {
                        "type": "string",
                        "description": "要获取拼音首字母的汉字文本，以“,”分隔"
                    }
                },
                "required": ["text"]
            }
        )

    async def run_tool(self, arguments: Dict[str, Any]) -> Sequence[TextContent]:
            """将中文文本转换为拼音首字母

            参数:
                text (str): 要转换的中文文本，以中文逗号分隔

            返回:
                list[TextContent]: 包含转换结果的TextContent列表
                - 每个词的首字母会被转换为大写
                - 多个词的结果以英文逗号连接

            示例:
                get_chinese_initials("用户名，密码")
                [TextContent(type="text", text="YHM,MM")]
            """
            try:
                if "text" not in arguments:
                    raise ValueError("缺少查询语句")

                text = arguments["text"]

                # 将文本按逗号分割
                words = text.split('，')

                # 存储每个词的首字母
                initials = []

                for word in words:
                    # 获取每个字的拼音首字母
                    word_pinyin = pinyin(word, style=Style.FIRST_LETTER)
                    # 将每个字的首字母连接起来
                    word_initials = ''.join([p[0].upper() for p in word_pinyin])
                    initials.append(word_initials)

                # 用逗号连接所有结果
                return [TextContent(type="text", text=','.join(initials))]

            except Exception as e:
                return [TextContent(type="text", text=f"执行查询时出错: {str(e)}")]


import re
from typing import Dict, Sequence, Any, List

from mcp import Tool
from mcp.types import TextContent

from .base import BaseHandler
from mysql_mcp_server_pro.config import get_db_config

from mysql_mcp_server_pro.exception.exceptions import SQLExecutionError
from ..utils.execute_sql_util import ExecuteSqlUtil

execute_sql = ExecuteSqlUtil()

class OptimizeSql(BaseHandler):
    name = "optimize_sql"
    description = (
        "专业的SQL性能优化工具，基于MySQL执行计划、表结构信息、表数据量、表索引提供专家级优化建议。"
        "该工具能够分析SQL语句的执行效率，识别性能瓶颈，并提供具体的优化方案，"
        "包括索引优化、查询重写建议等，帮助提升数据库查询性能。"
    )

    def get_tool_description(self) -> Tool:
        return Tool(
            name=self.name,
            description=self.description,
            inputSchema={
                "type": "object",
                "properties": {
                    "text": {
                        "type": "string",
                        "description": "要优化的sql"
                    }
                },
                "required": ["text"]
            }
        )

    async def run_tool(self, arguments: Dict[str, Any]) -> Sequence[TextContent]:
        try:
            if "text" not in arguments:
                raise ValueError("缺少查询语句")

            text = arguments["text"]

            config = get_db_config()

            tables = self.get_sql_all_tables(text)

            # 查询表的数据量
            table_count = self.get_tables_count(tables, config)

            # 查询表的索引情况
            table_index = self.get_tables_index(tables, config)

            # 获取sql的执行过程
            sql_explain = self.get_sql_execution_process(text, config)

            # 获取表的结构信息
            table_schemas = self.get_tables_schemas(tables, config)

            result = f"""
                        # 角色
                        你是一位世界级的数据库性能优化专家，拥有超过15年的SQL优化经验。你的专长包括：
                        1. 深入理解各种数据库的查询优化器工作原理（MySQL, PostgreSQL, Oracle, SQL Server等）
                        2. 精通执行计划分析和性能瓶颈识别
                        3. 擅长复杂查询重写和索引优化策略
                        4. 熟悉各种数据库特有的优化技巧和最佳实践
                        
                        # 背景
                        你的任务是分析用户提供的SQL语句，并提供专业级的优化建议。以下是相关信息：
                            <数据库类型>
                                mysql
                            </数据库类型>
                            <SQL语句>
                                {text}
                            </SQL语句>
                            <表结构信息>
                                {table_schemas}
                            </表结构信息>
                            <表索引信息>
                                {table_index}
                            </表索引信息>
                            <表数据量信息>
                                {table_count}
                            </表数据量信息>
                            <执行计划>
                                {sql_explain}
                            </执行计划>
                        
                        # 任务
                        请从以下几个方面进行分析，且每个建议都要说明问题的根本原因、优化的具体方法、预期的性能提升效果以及实施的注意事项和风险，同时要考虑业务场景和数据特征，提供最适合的优化方案：
                        1. **执行计划分析**
                           - 识别执行计划中的性能瓶颈
                           - 分析访问路径是否合理（全表扫描、索引扫描等）
                           - 评估连接顺序和连接方式的效率
                           - 检查是否存在不必要的排序和分组操作
                        
                        2. **SQL逻辑优化**
                           - 是否存在冗余或重复的查询逻辑
                           - WHERE条件是否能有效利用索引
                           - JOIN操作是否可以优化
                           - 子查询是否可以改写为JOIN或其他方式
                           - 是否存在可以简化的复杂表达式
                           - 查询的内容包含时间的是否可以增加时间范围
                           - 避免在WHERE、JOIN、ORDER BY中使用函数或表达式对字段进行操作
                           - 合理使用EXISTS替代IN（尤其在子查询中）
                           - 避免SELECT *，只查询需要的字段
                           - 使用LIMIT / TOP / FETCH 方式限制结果集
                           - 避免不必要的DISTINCT和GROUP BY
                           - 优化ORDER BY和分页逻辑
                           - 减少多层嵌套子查询
                           - 避免在WHERE中使用OR连接多个条件（尤其无索引字段）
                           - 使用覆盖索引（Covering Index）
                           - 使用分析函数替代自连接或子查询
                           - 避免隐式类型转换
                           - 合理使用UNION ALL替代UNION
                           - 避免 Cartesian Product（笛卡尔积）
                           - 使用 INNER JOIN 替代 WHERE 子句连接（隐式 JOIN）
                           - 合理使用 LEFT JOIN 和 INNER JOIN
                           - 避免在 JOIN 条件中使用函数或复杂表达式
                           - 使用物化临时表或 CTE 缓存中间结果
                           - 避免在 WHERE 中使用 != 或NOT IN
                           - 小结果集表做驱动表（通常是 LEFT 表）
                        
                        3. **索引优化建议**
                           - 现有索引的使用情况分析
                           - 是否需要创建新的复合索引
                           - 是否存在冗余或未使用的索引
                           - 索引列顺序是否合理
                           - 是否考虑使用覆盖索引
                        
                        4. **具体优化方案**
                           - 提供优化后的SQL语句，需要多个数据库版本的SQL优化方案
                           - 存在大表查询最新时间的查询时，需要提示能够缩小查询范围，提供优化后的SQL语句
                           - 若通过分析函数优化，窗口函数优化时需要考虑cte方式是否也能满足，比较两者输出最优方案
                           - 详细说明每项优化的预期效果
                           - 列出需要创建或修改的索引
                           - 估计优化后的性能提升幅度
                        
                        5. **实施建议和注意事项**
                           - 优化方案的实施步骤
                           - 可能遇到的风险和应对措施
                           - 验证优化效果的方法
                           - 在生产环境中实施的最佳时机
                        
                        此外，请扩展延伸以下情况：
                            1. 能否进一步分析这个查询在高并发场景下的表现？
                            2. 如果数据量增长100倍，这个优化方案是否仍然有效？sql又如何调整？
                            3. 除了SQL层面的优化，数据库参数层面有哪些可以调整的？
                            4. 这个优化方案对其他相似查询是否有借鉴意义？
                            5. 能否提供一个逐步实施的优化计划，以降低生产环境风险？
                            
                        最后，请以markdown格式输出结果，内容包括：
                            1. 描述相关表情况，表索引，表数据量
                            2. 解析执行计划的过程
                            3. 分析存在的问题点
                            4. 根据SQL逻辑优化提供解决方案，需要提供多个版本的方案，例如mysql5.7以下，以及mysql5.7、mysql8.0以上，需要提示适用哪些范围的版本,优化后的sql展示
                            5. 延伸思考点，若有新的sql优化思路，需要详细说明并进行展示
                            
                        """

            return [TextContent(type="text", text="".join( result))]


        except SQLExecutionError as e:
            return [TextContent(type="text", text=f"执行查询时出错: {str(e)}")]
        except Exception as e:
            return [TextContent(type="text", text=f"执行查询时出错: {str(e)}")]

    def get_sql_all_tables(self,sql: str) -> List[str]:
        """
        从SQL语句中提取所有涉及的表名

        Args:
            sql (str): SQL语句

        Returns:
            List[str]: 表名列表
        """
        # 移除SQL注释
        sql = re.sub(r'--.*$', '', sql, flags=re.MULTILINE)
        sql = re.sub(r'/\*.*?\*/', '', sql, flags=re.DOTALL)

        # 转换为大写以便处理
        sql_upper = sql.upper()

        # 首先提取CTE（公用表表达式）中的表名，这些不应被视为真实表
        cte_names = set()
        # 使用正则表达式直接匹配CTE定义 "name AS ("
        cte_pattern = r'(\w+)\s+AS\s*\('
        cte_matches = re.findall(cte_pattern, sql_upper)
        cte_names.update(cte_matches)

        # 匹配各种SQL语句中的表名
        table_names = set()

        # 匹配 FROM 子句中的表名 (包括 JOIN)
        from_patterns = [
            r'\bFROM\s+([^\s,]+)',
            r'\bJOIN\s+([^\s,]+)',
            r'\bUPDATE\s+([^\s,]+)',
            r'\bINTO\s+([^\s,]+)',
            r'\bTABLE\s+([^\s,]+)'
        ]

        for pattern in from_patterns:
            matches = re.findall(pattern, sql_upper, re.IGNORECASE)
            for match in matches:
                # 处理可能的别名和特殊字符
                table = match.split()[0].strip()
                # 移除可能的反引号、括号等
                table = re.sub(r'[`"\[\]()]', '', table)
                # 过滤掉一些关键字和表达式
                if (table and
                        not table.startswith('(') and
                        table not in cte_names and  # 排除CTE中的表名
                        table not in [
                            'SELECT', 'INSERT', 'UPDATE', 'DELETE', 'WHERE', 'GROUP', 'ORDER',
                            'HAVING', 'LIMIT', 'UNION', 'INTERSECT', 'EXCEPT', 'WITH'
                        ] and
                        not table.isdigit()):  # 排除纯数字
                    table_names.add(table)

        # 匹配 DELETE FROM 语句
        delete_matches = re.findall(r'\bDELETE\s+FROM\s+([^\s,]+)', sql_upper, re.IGNORECASE)
        for match in delete_matches:
            table = match.split()[0].strip()
            table = re.sub(r'[`"\[\]()]', '', table)
            if table and table not in cte_names:
                table_names.add(table)

        # 匹配 INSERT INTO 语句
        insert_matches = re.findall(r'\bINSERT\s+INTO\s+([^\s(]+)', sql_upper, re.IGNORECASE)
        for match in insert_matches:
            table = match.split()[0].strip()
            table = re.sub(r'[`"\[\]()]', '', table)
            if table and table not in cte_names:
                table_names.add(table)

        # 匹配 REPLACE INTO 语句
        replace_matches = re.findall(r'\bREPLACE\s+INTO\s+([^\s(]+)', sql_upper, re.IGNORECASE)
        for match in replace_matches:
            table = match.split()[0].strip()
            table = re.sub(r'[`"\[\]()]', '', table)
            if table and table not in cte_names:
                table_names.add(table)

        return list(table_names)

    def get_tables_index(self,tables,config) -> str:

        table_condition = "','".join(tables)

        sql = "SELECT TABLE_NAME, INDEX_NAME, COLUMN_NAME, SEQ_IN_INDEX, NON_UNIQUE, INDEX_TYPE "
        sql += f"FROM information_schema.STATISTICS WHERE TABLE_SCHEMA = '{config['database']}' "
        sql += f"AND TABLE_NAME IN ('{table_condition}') ORDER BY TABLE_NAME, INDEX_NAME, SEQ_IN_INDEX;"

        return execute_sql.format_result(execute_sql.execute_single_statement(sql))

    def get_tables_count(self, tables,config) -> str:
        table_condition = "','".join(tables)

        sql = f"""
            SELECT 
                table_name AS `Table`, 
                round(((data_length + index_length) / 1024 / 1024), 2) AS `Size (MB)`
            FROM 
                information_schema.tables
            WHERE 
                table_schema = '{config['database']}'
                AND table_name in ('{table_condition}')  
            ORDER BY 
                (data_length + index_length) DESC;

        """

        return execute_sql.format_result(execute_sql.execute_single_statement(sql))

    def get_sql_execution_process(self, text, config) -> str:
        sql = "EXPLAIN " + text

        return execute_sql.format_result(execute_sql.execute_single_statement(sql))

    def get_tables_schemas(self, tables,config) -> str:
        table_condition = "','".join(tables)
        sql = "SELECT TABLE_NAME, COLUMN_NAME, COLUMN_COMMENT "
        sql += f"FROM information_schema.COLUMNS WHERE TABLE_SCHEMA = '{config['database']}' "
        sql += f"AND TABLE_NAME IN ('{table_condition}') ORDER BY TABLE_NAME, ORDINAL_POSITION;"
        return execute_sql.format_result(execute_sql.execute_single_statement(sql))




from typing import Dict, Any

from mcp import GetPromptResult
from mcp.types import Prompt, TextContent, PromptMessage, PromptArgument

from mysql_mcp_server_pro.prompts.BasePrompt import BasePrompt


class QueryTableData(BasePrompt):
    name = "query-table-data-prompt"
    description = (
        "这是通过调用工具查询表数据的提示词"
    )

    def get_prompt(self) -> Prompt:
        return Prompt(
            name= self.name,
            description= self.description,
            arguments=[
                PromptArgument(
                    name="desc", description="请输入需要查询的内容,为空时大模型会初始化为数据库助手"
                )
            ],
        )

    async def run_prompt(self, arguments: Dict[str, Any]) -> GetPromptResult:

        prompt = """
        - Role: 数据库应用开发专家和自然语言处理工程师
        - Background: 目前有一个操作MySQL数据库的MCP Server，该服务器具备根据表描述获取数据库表名和根据表名获取表结构的功能。用户希望实现一个自然语言交互功能，通过用户输入的自然语言指令，如“查询用户表张三的数据”，自动调用相关工具完成任务。
        - Profile: 你是一位数据库应用开发专家，同时具备自然语言处理的技能，能够理解用户输入的自然语言指令，并将其转化为数据库操作的具体步骤。你熟悉MySQL数据库的操作，以及如何通过编程接口调用相关工具。
        - Skills: 你具备自然语言解析、数据库查询、API调用、逻辑推理等关键能力，能够将用户的自然语言指令转化为具体的数据库操作流程。
        - Goals: 根据用户输入的自然语言指令，自动调用“get_table_name”和“get_table_desc”的工具，最终实现用户期望的数据库操作。
        - Constrains: 该提示词应确保用户输入的自然语言指令能够被准确解析，调用的工具能够正确执行，且整个过程应具备良好的用户体验和错误处理机制。
        - OutputFormat: 以自然语言交互的形式输出，包括用户输入的解析结果、调用工具的中间结果以及最终的数据库查询结果。
        - Workflow:
          1. 解析用户输入的自然语言指令，提取关键信息，如表描述和查询条件。
          2. 调用“get_table_name”工具，获取对应的表名。
          3. 调用“get_table_desc”工具，获取表的结构信息。
          4. 根据表结构信息和用户输入的查询条件，生成SQL查询语句并调用“execute_sql”工具，返回查询结果。
        - Examples:
          - 例子1：用户输入“查询用户表张三的数据”
            解析结果：表描述为“用户表”，查询条件为“张三”。
            调用工具1：根据“用户表”描述获取表名，假设返回表名为“user_table”。
            调用工具2：根据“user_table”获取表结构，假设表结构包含字段“id”、“name”、“age”。
            生成SQL查询语句：`SELECT * FROM user_table WHERE name = '张三';`
            调用工具3：根据生成的SQL,获取结果。
            查询结果：返回张三的相关数据。
          - 例子2：用户输入“查询商品表价格大于100的商品”
            解析结果：表描述为“商品表”，查询条件为“价格大于100”。
            调用工具1：根据“商品表”描述获取表名，假设返回表名为“product_table”。
            调用工具2：根据“product_table”获取表结构，假设表结构包含字段“id”、“name”、“price”。
            生成SQL查询语句：`SELECT * FROM product_table WHERE price > 100;`
            调用工具3：根据生成的SQL,获取结果。
            查询结果：返回价格大于100的商品数据。
          - 例子3：用户输入“查询订单表张三的订单金额”
            解析结果：表描述为“订单表”，查询条件为“张三”。
            调用工具1：根据“订单表”描述获取表名，假设返回表名为“order_table”。
            调用工具2：根据“order_table”获取表结构，假设表结构包含字段“id”、“user_name”、“order_amount”。
            生成SQL查询语句：`SELECT order_amount FROM order_table WHERE user_name = '张三';`
            调用工具3：根据生成的SQL,获取结果。
            查询结果：返回张三的订单金额。       
        """

        if "desc" not in arguments:
            prompt += """- Initialization: 在第一次对话中，请直接输出以下：您好，作为数据库应用开发专家，我将协助您实现自然语言交互功能。
            请按照以下格式输入您的需求：“查询[表描述][查询条件]的数据”，例如“查询用户表张三的数据”，我将为您自动调用相关工具并返回查询结果。"""
        else:
            desc = arguments["desc"]
            prompt += f"- task: {desc}。   "

        return GetPromptResult(
            description="mysql prompt",
            messages=[
                PromptMessage(
                    role="user",
                    content=TextContent(type="text", text=prompt),
                )
            ],
        )


"""
SQL执行工具类，使用数据库连接池执行SQL语句
"""

import logging
import re
from enum import Enum
from typing import List, Tuple, Optional, Dict, Any, Set
from dataclasses import dataclass
from contextlib import contextmanager

from mysql.connector import Error as MySQLError

from .database_pool import create_mysql_pool
from ..config import get_db_config, get_role_permissions
from ..exception.exceptions import SQLPermissionError

logger = logging.getLogger(__name__)

class SQLOperation(str, Enum):
    """SQL 操作类型枚举"""
    SELECT = 'SELECT'
    INSERT = 'INSERT'
    UPDATE = 'UPDATE'
    DELETE = 'DELETE'
    CREATE = 'CREATE'
    ALTER = 'ALTER'
    DROP = 'DROP'
    TRUNCATE = 'TRUNCATE'
    SHOW = 'SHOW'
    DESCRIBE = 'DESCRIBE'
    EXPLAIN = 'EXPLAIN'

    @classmethod
    def from_str(cls, value: str) -> 'SQLOperation':
        """从字符串创建 SQLOperation 枚举值

        Args:
            value: SQL 操作字符串

        Returns:
            SQLOperation: 对应的枚举值

        Raises:
            ValueError: 当操作类型不支持时
        """
        try:
            return cls(value.upper())
        except ValueError:
            raise ValueError(f"不支持的SQL操作类型: {value}")

@dataclass
class SQLResult:
    """SQL执行结果"""
    success: bool
    message: str
    columns: Optional[List[str]] = None
    rows: Optional[List[Tuple]] = None
    affected_rows: int = 0


class ExecuteSqlUtil:
    """使用数据库连接池的SQL执行工具类"""

    # SQL操作正则模式
    SQL_COMMENT_PATTERN = re.compile(r'--.*$|/\*.*?\*/', re.MULTILINE | re.DOTALL)

    # 类级别的连接池，确保单例
    _connection_pool = None

    @classmethod
    def create_mysql_pool(cls, db_config: Dict[str, Any]):
        # 提取连接池相关配置
        pool_config = {
            'pool_size': db_config.get('pool_size', 10),
            'max_overflow': db_config.get('max_overflow', 20),
            'pool_recycle': db_config.get('pool_recycle', 3600),
            'pool_timeout': db_config.get('pool_timeout', 30)
        }

        # 提取数据库连接配置
        db_params = {
            'host': db_config.get('host', 'localhost'),
            'port': db_config.get('port', 3306),
            'user': db_config.get('user', 'root'),
            'password': db_config.get('password', ''),
            'database': db_config.get('database', '')
        }

        # 创建MySQL连接池
        cls._connection_pool = create_mysql_pool(
            host=db_params['host'],
            port=db_params['port'],
            user=db_params['user'],
            password=db_params['password'],
            database=db_params['database'],
            pool_size=pool_config['pool_size'],
            max_overflow=pool_config['max_overflow'],
            pool_recycle=pool_config['pool_recycle'],
            pool_timeout=pool_config['pool_timeout']
        )

    @classmethod
    @contextmanager
    def get_db_connection(cls):
        """获取数据库连接的上下文管理器
        
        Yields:
            数据库连接对象
        """
        connection = None
        try:

            if cls._connection_pool is None:
                cls.create_mysql_pool(db_config=get_db_config())

            connection = cls._connection_pool.get_connection()
            yield connection
        except Exception as e:
            logger.error(f"从连接池获取数据库连接失败: {e}")
            raise
        finally:
            if connection is not None:
                try:
                    cls._connection_pool.return_connection(connection)
                except Exception as e:
                    logger.warning(f"归还数据库连接到连接池时出错: {e}")

    def execute_single_statement(self, statement: str) -> SQLResult:
        """执行单条SQL语句
        
        Args:
            statement: SQL语句
            
        Returns:
            SQL执行结果
            
        Raises:
            Exception: 当执行出错时
        """
        try:
            # 检查权限
            operations = self.extract_operations(statement)
            self.check_permissions(operations)

            with ExecuteSqlUtil.get_db_connection() as conn:
                from sqlalchemy import text
                
                # 清理SQL语句并转为大写进行分析
                cleaned_statement = self.clean_sql(statement)
                upper_statement = cleaned_statement.upper().strip()
                
                # 判断语句类型
                is_select = (upper_statement.startswith('SELECT') or 
                           upper_statement.startswith('WITH'))
                
                is_show = upper_statement.startswith('SHOW')
                is_explain = upper_statement.startswith('EXPLAIN')
                is_describe = (upper_statement.startswith('DESCRIBE') or 
                             upper_statement.startswith('DESC '))
                
                # 特殊语句类型（通常返回结果集）
                is_query_type = is_select or is_show or is_explain or is_describe
                
                try:
                    # 执行SQL语句
                    result = conn.execute(text(statement))
                    
                    # 根据语句类型处理结果
                    if is_query_type:
                        # 查询类语句（SELECT, SHOW, EXPLAIN, DESCRIBE等）
                        columns = list(result.keys())
                        rows = result.fetchall()
                        return SQLResult(
                            success=True,
                            message="查询执行成功",
                            columns=columns,
                            rows=rows
                        )
                    else:
                        # 非查询语句（INSERT, UPDATE, DELETE等）
                        conn.commit()
                        return SQLResult(
                            success=True,
                            message="执行成功",
                            affected_rows=result.rowcount
                        )
                except Exception as e:
                    # 如果是非查询语句且执行失败，回滚事务
                    if not is_query_type:
                        conn.rollback()
                    raise
                        
        except MySQLError as e:
            logger.error(f"SQL执行错误: {e}, SQL: {statement}")
            return SQLResult(
                success=False,
                message=f"执行失败: {str(e)}"
            )
        except Exception as e:
            logger.error(f"未知错误: {e}, SQL: {statement}")
            return SQLResult(
                success=False,
                message=f"执行失败: {str(e)}"
            )

    def execute_multiple_statements(self, query: str) -> List[SQLResult]:
        """执行多条SQL语句
        
        Args:
            query: 包含多条SQL语句的查询字符串，以分号分隔
            
        Returns:
            SQL执行结果列表
        """
        statements = [stmt.strip() for stmt in query.split(';') if stmt.strip()]
        results = []
        
        for statement in statements:
            try:
                result = self.execute_single_statement(statement)
                results.append(result)
            except Exception as e:
                logger.warning(f"SQL执行警告: {e}, SQL: {statement}")
                results.append(SQLResult(
                    success=False,
                    message=f"执行失败: {str(e)}"
                ))
                
        return results

    def get_connection_pool_stats(self) -> Dict[str, Any]:
        """获取连接池统计信息
        
        Returns:
            连接池统计信息
        """
        return self._connection_pool.get_stats()

    def _get_allowed_operations(self) -> Set[SQLOperation]:
        """获取当前角色允许的操作列表

        Returns:
            Set[SQLOperation]: 允许的操作集合
        """
        config = get_db_config()
        role = config.get("role", "readonly")  # 默认为只读角色
        return {SQLOperation.from_str(op) for op in get_role_permissions(role)}

    def clean_sql(self, sql: str) -> str:
        """清理SQL语句，移除注释和多余空白

        Args:
            sql: 原始SQL语句

        Returns:
            清理后的SQL语句
        """
        # 移除注释
        sql = self.SQL_COMMENT_PATTERN.sub('', sql)
        # 规范化空白字符
        return ' '.join(sql.split())

    def extract_operations(self, sql: str) -> Set[SQLOperation]:
        """提取SQL语句中的所有操作类型

        Args:
            sql: SQL语句

        Returns:
            操作类型集合
        """
        sql = self.clean_sql(sql.upper())
        return {
            op for op in SQLOperation
            if re.search(rf'\b{op.value}\b', sql)
        }

    def check_permissions(self, operations: Set[SQLOperation]) -> bool:
        """检查操作权限

        Args:
            operations: 操作类型集合

        Returns:
            是否有权限执行所有操作

        Raises:
            SQLPermissionError: 当权限不足时
        """
        allowed = self._get_allowed_operations()
        unauthorized = operations - allowed

        if unauthorized:
            raise SQLPermissionError(
                f"权限不足: 当前角色无权执行以下操作: {', '.join(op.value for op in unauthorized)}"
            )
        return True

    def format_result(self, result: SQLResult) -> str:
        """格式化SQL执行结果

        Args:
            result: SQL执行结果

        Returns:
            格式化后的结果字符串
        """
        if not result.success:
            return result.message

        if result.columns and result.rows:  # SELECT 类查询结果
            # 将所有值转换为字符串，None转换为"NULL"
            formatted_rows = [
                ",".join("NULL" if v is None else str(v) for v in row)
                for row in result.rows
            ]
            return "\n".join([",".join(result.columns)] + formatted_rows)
        else:  # 非查询语句结果
            return f"{result.message}。影响行数: {result.affected_rows}"


import webbrowser
from starlette.middleware.base import BaseHTTPMiddleware
from starlette.requests import Request
from starlette.responses import JSONResponse
from typing import Optional, Dict, Set
import os
import time

from .token_handler import TokenHandler


class OAuthMiddleware(BaseHTTPMiddleware):
    """OAuth认证中间件"""
    
    # 类级别的弹窗记录，确保所有实例共享
    _global_popup_time: float = 0
    _popup_cooldown: float = 5  # 冷却时间（秒）
    
    def __init__(self, app, exclude_paths: Optional[list[str]] = None):
        """
        初始化中间件
        
        Args:
            app: Starlette应用实例
            exclude_paths: 不需要认证的路径列表
        """
        super().__init__(app)
        # 默认排除路径：登录相关页面和资源
        default_exclude_paths = [
            "/login",                    # 登录页面
            "/mcp/auth/login",          # 登录API
            "/favicon.ico",             # 网站图标
            "/static",                  # 静态资源
        ]
        self.exclude_paths = exclude_paths or default_exclude_paths
        self.login_url = os.getenv("MCP_LOGIN_URL", "http://localhost:3000/login")
    
    def _is_excluded_path(self, path: str) -> bool:
        """
        检查路径是否在排除列表中
        
        Args:
            path: 请求路径
            
        Returns:
            bool: 是否排除认证
        """
        return any(
            path == excluded or path.startswith(f"{excluded}/")
            for excluded in self.exclude_paths
        )
    
    @classmethod
    def _should_show_popup(cls) -> bool:
        """
        判断是否需要显示弹窗
        
        Returns:
            bool: 是否需要显示弹窗
        """
        current_time = time.time()
        
        # 检查是否在冷却期内
        if current_time - cls._global_popup_time > cls._popup_cooldown:
            cls._global_popup_time = current_time
            return True
        return False
    
    async def dispatch(self, request: Request, call_next):
        """
        处理请求
        
        Args:
            request: 请求对象
            call_next: 下一个处理函数
        """
        # 检查是否需要跳过认证
        if self._is_excluded_path(request.url.path):
            return await call_next(request)
            
        # 获取认证头
        auth_header = request.headers.get("Authorization")
        if not auth_header:
            # 只在需要时弹出登录框，并且不是API请求时
            if (not request.url.path.startswith("/api/") and 
                not request.headers.get("accept", "").startswith("application/json") and
                self._should_show_popup()):
                webbrowser.open(self.login_url)
            
            return JSONResponse(
                {"error": "invalid_request", "error_description": "Missing authorization header"},
                status_code=401
            )
            
        # 验证token格式
        parts = auth_header.split()
        if len(parts) != 2 or parts[0].lower() != "bearer":
            return JSONResponse(
                {"error": "invalid_request", "error_description": "Invalid authorization header format"},
                status_code=401
            )
            
        token = parts[1]
        
        # 验证token
        payload = TokenHandler.verify_token(token)
        if not payload:
            return JSONResponse(
                {"error": "invalid_token", "error_description": "Token is invalid or expired"},
                status_code=401
            )
            
        # 检查token类型
        if payload.get("type") != "access_token":
            return JSONResponse(
                {"error": "invalid_token", "error_description": "Invalid token type"},
                status_code=401
            )
            
        # 将用户信息添加到请求对象
        request.state.user = {
            "id": payload["sub"],
            "username": payload["username"]
        }
        
        return await call_next(request) 


from .config import oauth_config
from .token_handler import TokenHandler
from .middleware import OAuthMiddleware
from .routes import login, login_page

__all__ = [
    "oauth_config",
    "TokenHandler",
    "OAuthMiddleware",
    "login",
    "login_page"
] 


from typing import Dict, Any, Sequence

from mcp import Tool
from mcp.types import TextContent

from .base import BaseHandler

from mysql_mcp_server_pro.handles import (
    ExecuteSQL
)

execute_sql = ExecuteSQL()

class GetTableLock(BaseHandler):
    name = "get_table_lock"
    description = (
        "获取当前mysql服务器行级锁、表级锁情况(Check if there are row-level locks or table-level locks in the current MySQL server  )"
    )

    def get_tool_description(self) -> Tool:
        return Tool(
            name=self.name,
            description=self.description,
            inputSchema={
                "type": "object",
                "properties": {

                }
            }
        )

    async def run_tool(self, arguments: Dict[str, Any]) -> Sequence[TextContent]:
        use_result = await self.get_table_use(arguments)
        lock_result_5 = await self.get_table_lock_for_mysql5(arguments)
        lock_result_8 = await self.get_table_lock_for_mysql8(arguments)
        
        # 合并两个结果
        combined_result = []
        combined_result.extend(use_result)
        combined_result.extend(lock_result_5)
        combined_result.extend(lock_result_8)
        
        return combined_result

    """
        获取表级锁情况
    """
    async def get_table_use(self, arguments: Dict[str, Any]) -> Sequence[TextContent]:
        try:
            sql = "SHOW OPEN TABLES WHERE In_use > 0;"

            return await execute_sql.run_tool({"query": sql})
        except Exception as e:
            return [TextContent(type="text", text=f"执行查询时出错: {str(e)}")]

    """
        获取行级锁情况--mysql5.6
    """
    async def get_table_lock_for_mysql5(self, arguments: Dict[str, Any]) -> Sequence[TextContent]:
        try:
            sql = "SELECT p2.`HOST` 被阻塞方host,  p2.`USER` 被阻塞方用户, r.trx_id 被阻塞方事务id, "
            sql += "r.trx_mysql_thread_id 被阻塞方线程号,TIMESTAMPDIFF(SECOND, r.trx_wait_started, CURRENT_TIMESTAMP) 等待时间, "
            sql += "r.trx_query 被阻塞的查询, l.lock_table 阻塞方锁住的表, m.`lock_mode` 被阻塞方的锁模式, "
            sql += "m.`lock_type` '被阻塞方的锁类型(表锁还是行锁)', m.`lock_index` 被阻塞方锁住的索引, "
            sql += "m.`lock_space` 被阻塞方锁对象的space_id, m.lock_page 被阻塞方事务锁定页的数量, "
            sql += "m.lock_rec 被阻塞方事务锁定记录的数量, m.lock_data 被阻塞方事务锁定记录的主键值, "
            sql += "p.`HOST` 阻塞方主机, p.`USER` 阻塞方用户, b.trx_id 阻塞方事务id,b.trx_mysql_thread_id 阻塞方线程号, "
            sql += "b.trx_query 阻塞方查询, l.`lock_mode` 阻塞方的锁模式, l.`lock_type` '阻塞方的锁类型(表锁还是行锁)',"
            sql += "l.`lock_index` 阻塞方锁住的索引,l.`lock_space` 阻塞方锁对象的space_id,l.lock_page 阻塞方事务锁定页的数量,"
            sql += "l.lock_rec 阻塞方事务锁定行的数量,  l.lock_data 阻塞方事务锁定记录的主键值,"
            sql += "IF(p.COMMAND = 'Sleep', CONCAT(p.TIME, ' 秒'), 0) 阻塞方事务空闲的时间 "
            sql += "FROM information_schema.INNODB_LOCK_WAITS w "
            sql += "INNER JOIN information_schema.INNODB_TRX b ON b.trx_id = w.blocking_trx_id "
            sql += "INNER JOIN information_schema.INNODB_TRX r ON r.trx_id = w.requesting_trx_id "
            sql += "INNER JOIN information_schema.INNODB_LOCKS l ON w.blocking_lock_id = l.lock_id AND l.`lock_trx_id` = b.`trx_id` "
            sql += "INNER JOIN information_schema.INNODB_LOCKS m ON m.`lock_id` = w.`requested_lock_id` AND m.`lock_trx_id` = r.`trx_id` "
            sql += "INNER JOIN information_schema.PROCESSLIST p ON p.ID = b.trx_mysql_thread_id "
            sql += "INNER JOIN information_schema.PROCESSLIST p2 ON p2.ID = r.trx_mysql_thread_id "
            sql += "ORDER BY 等待时间 DESC;"

            return await execute_sql.run_tool({"query": sql})
        except Exception as e:
            return [TextContent(type="text", text=f"执行查询时出错: {str(e)}")]

    """
        获取行级锁情况--mysql8
    """
    async def get_table_lock_for_mysql8(self, arguments: Dict[str, Any]) -> Sequence[TextContent]:
        try:
            sql = "SELECT p2.HOST AS '被阻塞方host',p2.USER AS '被阻塞方用户',r.trx_id AS '被阻塞方事务id', "
            sql += "r.trx_mysql_thread_id AS '被阻塞方线程号',TIMESTAMPDIFF(SECOND, r.trx_wait_started, CURRENT_TIMESTAMP) AS '等待时间',"
            sql += "r.trx_query AS '被阻塞的查询',dlr.OBJECT_SCHEMA AS '被阻塞方锁库',dlr.OBJECT_NAME AS '被阻塞方锁表',"
            sql += "dlr.LOCK_MODE AS '被阻塞方锁模式', dlr.LOCK_TYPE AS '被阻塞方锁类型',dlr.INDEX_NAME AS '被阻塞方锁住的索引',"
            sql += "dlr.LOCK_DATA AS '被阻塞方锁定记录的主键值',p.HOST AS '阻塞方主机',p.USER AS '阻塞方用户',b.trx_id AS '阻塞方事务id',"
            sql += "b.trx_mysql_thread_id AS '阻塞方线程号',b.trx_query AS '阻塞方查询',dlb.LOCK_MODE AS '阻塞方锁模式',"
            sql += "dlb.LOCK_TYPE AS '阻塞方锁类型',dlb.INDEX_NAME AS '阻塞方锁住的索引',dlb.LOCK_DATA AS '阻塞方锁定记录的主键值',"
            sql += "IF(p.COMMAND = 'Sleep', CONCAT(p.TIME, ' 秒'), 0) AS '阻塞方事务空闲的时间' "
            sql += "FROM performance_schema.data_lock_waits w "
            sql += "JOIN performance_schema.data_locks dlr ON w.REQUESTING_ENGINE_LOCK_ID = dlr.ENGINE_LOCK_ID "
            sql += "JOIN performance_schema.data_locks dlb ON w.BLOCKING_ENGINE_LOCK_ID = dlb.ENGINE_LOCK_ID "
            sql += "JOIN information_schema.innodb_trx r ON w.REQUESTING_ENGINE_TRANSACTION_ID = r.trx_id "
            sql += "JOIN information_schema.innodb_trx b ON w.BLOCKING_ENGINE_TRANSACTION_ID = b.trx_id "
            sql += "JOIN information_schema.processlist p ON b.trx_mysql_thread_id = p.ID "
            sql += "JOIN information_schema.processlist p2 ON r.trx_mysql_thread_id = p2.ID "
            sql += "ORDER BY '等待时间' DESC;"

            return await execute_sql.run_tool({"query": sql})
        except Exception as e:
            return [TextContent(type="text", text=f"执行查询时出错: {str(e)}")]